#ifndef CONTENT_H
#define CONTENT_H

#include <set>
#include <gvc.h>

#include "../graph.h"


struct PathParam {
	PathParam() { }
	PathParam(std::size_t n_close_dist, std::size_t n_far_dist, weight_t n_min_prob)
		: close_dist(n_close_dist), far_dist(n_far_dist), min_prob(n_min_prob) { }
	std::size_t close_dist;
	std::size_t far_dist;
	weight_t min_prob;
};

struct LayoutEdge {
	LayoutEdge() { }
	LayoutEdge(vertex_t n_tail, vertex_t n_head, weight_t n_weight)
		: tail(n_tail), head(n_head), weight(n_weight) { }
	vertex_t tail;
	vertex_t head;
	weight_t weight;
};

struct ltedge {
  bool operator()(const LayoutEdge e1, const LayoutEdge e2) const {
    if (e1.tail < e2.tail) return true;
	else if (e1.tail > e2.tail) return false;
	else return (e1.head < e2.head);
  }
};

class Content {

public:
	Content() { }

	void AddPath(const std::vector<vertex_t> path, const PathParam params) { 
		paths[path] = params; std::cout << "addPath" << std::endl; }
	void ModPath(const std::vector<vertex_t> path, const PathParam params) { 
		if (paths.find(path) != paths.end()) { paths[path] = params; std::cout << "modPath" << std::endl; }
	}
	void DelPath(const std::vector<vertex_t> path) { paths.erase(path); std::cout << "delPath" << std::endl; }
	PathParam* GetPathParam(const std::vector<vertex_t> path) { 
		if (paths.find(path) != paths.end()) return &(paths[path]); else return NULL;
	}
	size_t GetNumberPaths() const { return paths.size(); }

//	void AddNode(const vertex_t node);
//	void DelNode(const vertex_t node);
	Agraph_t* ComputeLayout(Graph& g, const std::vector<vertex_t> add_vertices, const PathParam add_params) const;
	Agraph_t* LayoutAlgo(const std::set<LayoutEdge, ltedge>& edges,  const std::vector<vertex_t>& add_vertices, const Graph& g) const;
	
private:
	std::map<std::vector<vertex_t>, PathParam> paths;
	std::set<vertex_t> added_nodes;
	std::set<vertex_t> deleted_nodes;
};

#endif // CONTENT_H
