#include <fstream>
#include <sstream>
#include <iostream>
#include <set>
#include <string>
#include <vector>

#include <math.h>
#include <stdio.h>
#include <time.h>

#include <wx/mimetype.h>

#ifdef __unix__
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif

#include "guiFrm.h"
#include "guiApp.h"
#include "loadNetFrm.h"
#include "optionFrm.h"
#include "paramset.h"
#include "../debug.h"
#include "../graph.h"

//#ifdef __WXMSW__
//    #define USE_XPM_BITMAPS 0
//#else
    #define USE_XPM_BITMAPS 1
//#endif

#if USE_XPM_BITMAPS && defined(__WXMSW__) && !wxUSE_XPM_IN_MSW
//    #error You need to enable XPM support to use XPM bitmaps with toolbar!
#endif // USE_XPM_BITMAPS

//#if !defined(__WXMSW__) && !defined(__WXPM__)
//    #include "mondrian.xpm"
//#endif

#if USE_XPM_BITMAPS
    #include "bitmaps/open.xpm"
    #include "bitmaps/save.xpm"
	#include "bitmaps/savegraph.xpm"
    #include "bitmaps/find.xpm"
    #include "bitmaps/add.xpm"
    #include "bitmaps/del.xpm"
    #include "bitmaps/zoom.xpm"
    #include "bitmaps/zoomin.xpm"
    #include "bitmaps/zoomout.xpm"
#endif // USE_XPM_BITMAPS

std::size_t peak_mem_usage;


////Event Table Start
BEGIN_EVENT_TABLE(guiFrm,wxFrame)
	EVT_MENU(wxID_EXIT,guiFrm::OnMenuExit)
	EVT_MENU(ID_MNU_HELPCONTENT,guiFrm::OnHelpContent)
	EVT_MENU(ID_MNU_ABOUT,guiFrm::OnAbout)
	EVT_MENU(ID_MNU_OPTION,guiFrm::OnMenuOptions)
	EVT_MENU(ID_MNU_DELRESULTTAB,guiFrm::OnBtnDelResultTabClick)
	EVT_MENU(ID_MNU_LOADGRAPH,guiFrm::OnBtnLoadGraphClick)
	EVT_MENU(ID_MNU_REDRAW,guiFrm::OnMenuRedraw)
	EVT_MENU(ID_MNU_SPLIT_LR,guiFrm::OnMenuSplitLR)
	EVT_MENU(ID_MNU_SPLIT_TB,guiFrm::OnMenuSplitTB)

	EVT_TOOL(wxID_SAVE,guiFrm::OnBtnSaveResultsClick)
	EVT_TOOL(wxID_OPEN,guiFrm::OnBtnLoadResultsClick)
	EVT_TOOL(wxID_PRINT,guiFrm::OnBtnPrintGraphClick)
	EVT_TOOL(ID_BTN_SAVEGRAPH,guiFrm::OnBtnExportGraphClick)
	EVT_TOOL(ID_BTN_ADDGRAPHTAB,guiFrm::OnBtnAddGraphTabClick)
	EVT_TOOL(ID_BTN_DELGRAPHTAB,guiFrm::OnBtnDelGraphTabClick)
	EVT_TOOL(ID_BTN_ZOOMFIT,guiFrm::OnBtnZoomFitClick)
	EVT_TOOL(ID_BTN_ZOOMOUT,guiFrm::OnBtnZoomOutClick)
	EVT_TOOL(ID_BTN_ZOOMIN,guiFrm::OnBtnZoomInClick)
	EVT_COMBOBOX(ID_CBX_ZOOM, guiFrm::OnCbxZoomChange)

	EVT_BUTTON(ID_BTN_LOADGRAPH,guiFrm::OnBtnLoadGraphClick)
	
	EVT_BUTTON(ID_BTN_ADDSTARTNODE,guiFrm::OnBtnAddStartNodeClick)
	EVT_BUTTON(ID_BTN_LOADSTARTNODEFILE,guiFrm::OnBtnLoadStartNodeFileClick)
	EVT_BUTTON(ID_BTN_DELSTARTNODE,guiFrm::OnBtnDelStartNodeClick)
	EVT_TEXT_ENTER(ID_EDT_STARTNODE,guiFrm::OnBtnAddStartNodeClick)

	EVT_BUTTON(ID_BTN_ADDENDNODE,guiFrm::OnBtnAddEndNodeClick)
	EVT_BUTTON(ID_BTN_LOADENDNODEFILE,guiFrm::OnBtnLoadEndNodeFileClick)
	EVT_BUTTON(ID_BTN_DELENDNODE,guiFrm::OnBtnDelEndNodeClick)
	EVT_TEXT_ENTER(ID_EDT_ENDNODE,guiFrm::OnBtnAddEndNodeClick)

	EVT_BUTTON(ID_BTN_SEARCHPATHS,guiFrm::OnBtnSearchPathsClick)
	EVT_BUTTON(ID_BTN_STOPSEARCH,guiFrm::OnBtnStopSearchClick)

	EVT_BUTTON(ID_BTN_ADDRESULTTAB,guiFrm::OnBtnAddResultTabClick)
	EVT_BUTTON(ID_BTN_DELRESULTTAB,guiFrm::OnBtnDelResultTabClick)

	EVT_NOTEBOOK_PAGE_CHANGED(ID_NTB_GRAPHS,guiFrm::OnNtbGraphsTabChanged)
	EVT_NOTEBOOK_PAGE_CHANGED(ID_NTB_RESULTS,guiFrm::OnNtbResultsTabChanged)
	EVT_CLOSE(guiFrm::guiFrmClose)
	
	EVT_COMMAND(wxID_ANY, EVT_ADD_GRAPH_TAB, guiFrm::OnAddGraphTab)
	EVT_COMMAND(wxID_ANY, EVT_ADD_RESLIST_TAB, guiFrm::OnAddResListTab)
END_EVENT_TABLE()
////Event Table End

guiFrm::guiFrm(wxWindow *parent, wxWindowID id, const wxString &title, const wxPoint &position, const wxSize& size, long style)
: wxFrame(parent, id, title, position, size, style) {
	OpenFileDialog =  new wxFileDialog(this, wxT("Choose a file"), wxT(""), wxT(""), wxT("*.*"), wxOPEN);
	search_status = 0;
	graphTabNo = 0;
	resTabNo = 0;

	debug.turn_off();
	info.turn_off();

	CreateControls();
	CreateToolBar();
	CreateMenu();
	CreateStatusBar();
	ManageWindow();

	loadNet = new loadNetFrm(NULL);
}

void guiFrm::CreateControls(void) {
// Main window begin
	wxBoxSizer* boxsizer_Frame = new wxBoxSizer(wxVERTICAL);
	this->SetSizer(boxsizer_Frame);
	this->SetAutoLayout(TRUE);

	SplitterWindow_TopBottom = new wxSplitterWindow(this, ID_SPLITTERWINDOW_TOPBOTTOM, wxDefaultPosition, wxDefaultSize);
	SplitterWindow_TopBottom->SetMinimumPaneSize(1);
	SplitterWindow_TopBottom->SetSashGravity(1);
	boxsizer_Frame->Add(SplitterWindow_TopBottom, 1, wxEXPAND | wxALL, 0);
	boxsizer_Frame->SetMinSize (wxSize(700,600));

	SplitterWindow_LeftRight = new wxSplitterWindow(SplitterWindow_TopBottom, ID_SPLITTERWINDOW_LEFTRIGHT, wxDefaultPosition, wxDefaultSize);
	SplitterWindow_LeftRight->SetMinimumPaneSize(1);

	wnd_Left = new wxScrolledWindow(SplitterWindow_LeftRight, ID_WND_LEFT, wxDefaultPosition, wxDefaultSize, wxVSCROLL | wxHSCROLL);
	wxBoxSizer* boxsizer_Left = new wxBoxSizer(wxVERTICAL);
	wnd_Left->SetSizer(boxsizer_Left);
	wnd_Left->SetAutoLayout(TRUE);

	wnd_Right = new wxScrolledWindow(SplitterWindow_LeftRight, ID_WND_RIGHT, wxDefaultPosition, wxDefaultSize, wxVSCROLL | wxHSCROLL);
	wxBoxSizer* boxsizer_Right = new wxBoxSizer(wxVERTICAL);
	wnd_Right->SetSizer(boxsizer_Right);
	wnd_Right->SetAutoLayout(TRUE);

	wnd_Bottom = new wxScrolledWindow(SplitterWindow_TopBottom, ID_WND_BOTTOM, wxDefaultPosition, wxDefaultSize, wxVSCROLL | wxHSCROLL);
	wxBoxSizer* boxsizer_Bottom = new wxBoxSizer(wxVERTICAL);
	wnd_Bottom->SetSizer(boxsizer_Bottom);
	wnd_Bottom->SetAutoLayout(TRUE);
	//boxsizer_Bottom->SetMinSize(1,150);

	SplitterWindow_LeftRight->SplitVertically(wnd_Left, wnd_Right, 300);
	SplitterWindow_TopBottom->SplitHorizontally(SplitterWindow_LeftRight, wnd_Bottom, -150);
// Main window end

// OptInfo-Tabbing begin
	ntb_OptInfo = new wxNotebook(wnd_Left, ID_NTB_OPTINFO, wxDefaultPosition, wxDefaultSize);
	boxsizer_Left->Add(ntb_OptInfo, 1, wxGROW | wxALL, 5);

	wxPanel *optPanel = new wxPanel(ntb_OptInfo);
	ntb_OptInfo->AddPage(optPanel, wxT("Parameters"), true);

	wxPanel *infoPanel = new wxPanel(ntb_OptInfo);
	ntb_OptInfo->AddPage(infoPanel, wxT("Information"), true);
	infoPanelSizer = new wxBoxSizer(wxVERTICAL);
	infoPanel->SetSizer(infoPanelSizer);
	infoPanel->SetAutoLayout(TRUE);

	ntb_OptInfo->SetSelection(0);

	// Option Tab begin
		wxBoxSizer* boxsizer_OptPanel = new wxBoxSizer(wxVERTICAL);
		optPanel->SetSizer(boxsizer_OptPanel);
		optPanel->SetAutoLayout(TRUE);

		ntb_Options = new wxNotebook(optPanel, ID_NTB_OPTIONS, wxDefaultPosition, wxDefaultSize);
		boxsizer_OptPanel->Add(ntb_Options, 1, wxGROW | wxALL, 5);
		
		wxPanel *optPanelMain = new wxPanel(ntb_Options);
		ntb_Options->AddPage(optPanelMain, wxT("Main"), true);

		optPanelMainSizer = new wxBoxSizer(wxVERTICAL);
		optPanelMain->SetSizer(optPanelMainSizer);
		optPanelMain->SetAutoLayout(TRUE);

		wxPanel *optPanelStart = new wxPanel(ntb_Options);
		ntb_Options->AddPage(optPanelStart, wxT("Start vertices"), true);
		wxBoxSizer* optPanelStartSizer = new wxBoxSizer(wxVERTICAL);
		optPanelStart->SetSizer(optPanelStartSizer);
		optPanelStart->SetAutoLayout(TRUE);

		wxPanel *optPanelEnd = new wxPanel(ntb_Options);
		ntb_Options->AddPage(optPanelEnd, wxT("End vertices"), true);
		wxBoxSizer* optPanelEndSizer = new wxBoxSizer(wxVERTICAL);
		optPanelEnd->SetSizer(optPanelEndSizer);
		optPanelEnd->SetAutoLayout(TRUE);

		ntb_Options->SetSelection(0);
		
		// Option Main begin
		btn_LoadGraph = new wxButton(optPanelMain, ID_BTN_LOADGRAPH, wxT("Load Network"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_LoadGraph"));
		if (wxGetApp().modIsSPD) btn_LoadGraph->SetToolTip(wxT("Loads the network file to be searched in"));
		else {
			btn_LoadGraph->SetLabel(wxT("Load Graph"));
			btn_LoadGraph->SetToolTip(wxT("Loads the graph file to be searched in"));
		}
		optPanelMainSizer->Add(btn_LoadGraph,0,wxALIGN_LEFT | wxALL,5);

		edt_GraphFile = new wxTextCtrl(optPanelMain, ID_EDT_GRAPHFILE, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_READONLY, wxDefaultValidator, wxT("edt_GraphFile"));
		edt_GraphFile->SetToolTip(wxT("Filename and -path of the currently loaded network file."));
		optPanelMainSizer->Add(edt_GraphFile, 0, wxGROW | wxALL, 5);

		wxFlexGridSizer* GridSizer_Opt1 = new wxFlexGridSizer(4, 3, 0, 0);
		optPanelMainSizer->Add(GridSizer_Opt1, 0, wxGROW | wxALL, 5);

		stt_Pathlength = new wxStaticText(optPanelMain, ID_STT_PATHLENGTH, wxT("Path length"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("stt_Pathlength"));
		GridSizer_Opt1->Add(stt_Pathlength, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		edt_Pathlength = new wxTextCtrl(optPanelMain, ID_EDT_PATHLENGTH, wxT("8"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("edt_Pathlength"));
		edt_Pathlength->SetToolTip(wxT("The length of the requested paths."));
		GridSizer_Opt1->Add(edt_Pathlength, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		wxStaticText *stt_Empty1 = new wxStaticText(optPanelMain, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxT("stt_Empty1"));
		GridSizer_Opt1->Add(stt_Empty1, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

		stt_NumPaths = new wxStaticText(optPanelMain, ID_STT_NUMPATHS, wxT("Number of paths"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("stt_NumPaths"));
		GridSizer_Opt1->Add(stt_NumPaths, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		edt_NumPaths = new wxTextCtrl(optPanelMain, ID_EDT_NUMPATHS, wxT("50"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("edt_NumPaths"));
		edt_NumPaths->SetToolTip(wxT("Number of paths in the result list."));
		GridSizer_Opt1->Add(edt_NumPaths, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		wxStaticText *stt_Empty2 = new wxStaticText(optPanelMain, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxT("stt_Empty2"));
		GridSizer_Opt1->Add(stt_Empty2, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

		stt_Filter = new wxStaticText(optPanelMain, ID_STT_FILTER, wxT("Filter"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("stt_Filter"));
		GridSizer_Opt1->Add(stt_Filter, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		edt_Filter = new wxTextCtrl(optPanelMain, ID_EDT_FILTER, wxT("70"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("edt_Filter"));
		edt_Filter->SetToolTip(wxT("Maximal similarity of two paths in a result list."));
		GridSizer_Opt1->Add(edt_Filter, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		wxStaticText *stt_Percent1 = new wxStaticText(optPanelMain, -1, wxT("%"), wxDefaultPosition, wxDefaultSize, 0, wxT("stt_Percent1"));
		GridSizer_Opt1->Add(stt_Percent1, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

		stt_SuccessProb = new wxStaticText(optPanelMain, ID_STT_SUCCESSPROB, wxT("Success\nprobability"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("stt_SuccessProb"));
		GridSizer_Opt1->Add(stt_SuccessProb, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		edt_SuccessProb = new wxTextCtrl(optPanelMain, ID_EDT_SUCCESSPROB, wxT("99.9"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("edt_SuccessProb"));
		edt_SuccessProb->SetToolTip(wxT("Probability that the best found path is really the best path."));
		GridSizer_Opt1->Add(edt_SuccessProb, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		wxStaticText *stt_Percent2 = new wxStaticText(optPanelMain, -1, wxT("%"), wxDefaultPosition, wxDefaultSize, 0, wxT("stt_Percent2"));
		GridSizer_Opt1->Add(stt_Percent2, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
		// Option Main end

		// Option start vertices begin
		rbt_AllStartNodes = new wxRadioButton(optPanelStart, ID_RBT_ALLSTARTNODES, wxT("All vertices"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
		optPanelStartSizer->Add(rbt_AllStartNodes, 0, wxGROW | wxALL, 2);
		rbt_SomeStartNodes = new wxRadioButton(optPanelStart, ID_RBT_SOMESTARTNODES, wxT("Some vertices:"), wxDefaultPosition, wxDefaultSize);
		optPanelStartSizer->Add(rbt_SomeStartNodes, 0, wxGROW | wxALL, 2);

		wxArrayString arrayStringFor_lbx_StartNodes;
		lbx_StartNodes = new wxListBox(optPanelStart, ID_LBX_STARTNODES, wxDefaultPosition, wxDefaultSize, arrayStringFor_lbx_StartNodes, wxLB_EXTENDED);
		lbx_StartNodes->SetToolTip(wxT("List of all possible start vertices"));
		optPanelStartSizer->Add(lbx_StartNodes, 1, wxGROW | wxALL, 5);

		wxGridSizer* controlStartSizer = new wxGridSizer(2, 2, 0, 0);
		optPanelStartSizer->Add(controlStartSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

		btn_AddStartNode = new wxButton(optPanelStart, ID_BTN_ADDSTARTNODE, wxT("Add"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_AddStartNode"));
		btn_AddStartNode->SetToolTip(wxT("Adds the vertex in the text box to the list."));
		controlStartSizer->Add(btn_AddStartNode, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		edt_StartNode = new wxTextCtrl(optPanelStart, ID_EDT_STARTNODE, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER, wxDefaultValidator, wxT("edt_StartNode"));
		edt_StartNode->SetToolTip(wxT("Vertex which should be added to the list."));
		controlStartSizer->Add(edt_StartNode, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		btn_DelStartNode = new wxButton(optPanelStart, ID_BTN_DELSTARTNODE, wxT("Remove"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_DelStartNode"));
		btn_DelStartNode->SetToolTip(wxT("Removes the selected vertex/vertices from the list."));
		controlStartSizer->Add(btn_DelStartNode, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		btn_LoadStartNodeFile = new wxButton(optPanelStart, ID_BTN_LOADSTARTNODEFILE, wxT("File"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_LoadStartNodeFile"));
		btn_LoadStartNodeFile->SetToolTip(wxT("Loads the start vertices from a file."));
		controlStartSizer->Add(btn_LoadStartNodeFile, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);
		// Option start vertices end

		// Option end vertices begin
		rbt_AllEndNodes = new wxRadioButton(optPanelEnd, ID_RBT_ALLENDNODES, wxT("All vertices"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
		optPanelEndSizer->Add(rbt_AllEndNodes, 0, wxGROW | wxALL, 2);
		rbt_SomeEndNodes = new wxRadioButton(optPanelEnd, ID_RBT_SOMEENDNODES, wxT("Some vertices:"), wxDefaultPosition, wxDefaultSize);
		optPanelEndSizer->Add(rbt_SomeEndNodes, 0, wxGROW | wxALL, 2);

		wxArrayString arrayStringFor_lbx_EndNodes;
		lbx_EndNodes = new wxListBox(optPanelEnd, ID_LBX_ENDNODES, wxDefaultPosition, wxDefaultSize, arrayStringFor_lbx_EndNodes, wxLB_EXTENDED);
		lbx_EndNodes->SetToolTip(wxT("List of all possible end vertices"));
		optPanelEndSizer->Add(lbx_EndNodes, 1, wxGROW | wxALL, 5);

		wxGridSizer* controlEndSizer = new wxGridSizer(2, 2, 0, 0);
		optPanelEndSizer->Add(controlEndSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

		btn_AddEndNode = new wxButton(optPanelEnd, ID_BTN_ADDENDNODE, wxT("Add"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_AddEndNode"));
		btn_AddEndNode->SetToolTip(wxT("Add the vertex in the text box to the list"));
		controlEndSizer->Add(btn_AddEndNode, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		edt_EndNode = new wxTextCtrl(optPanelEnd, ID_EDT_ENDNODE, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER, wxDefaultValidator, wxT("edt_EndNode"));
		edt_EndNode->SetToolTip(wxT("Vertex which should be added to the list"));
		controlEndSizer->Add(edt_EndNode, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		btn_DelEndNode = new wxButton(optPanelEnd, ID_BTN_DELENDNODE, wxT("Remove"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_DelEndNode"));
		btn_DelEndNode->SetToolTip(wxT("Remove the selected vertex/vertices from the list"));
		controlEndSizer->Add(btn_DelEndNode, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

		btn_LoadEndNodeFile = new wxButton(optPanelEnd, ID_BTN_LOADENDNODEFILE, wxT("File"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT("btn_LoadEndNodeFile"));
		btn_LoadEndNodeFile->SetToolTip(wxT("Loads the end vertices from a file."));
		controlEndSizer->Add(btn_LoadEndNodeFile, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);
		// Option end vertices end
		EnableParameters(false);
	// Option Tab end

	// Information Tab begin
		edt_NodeEdgeInfo = new wxTextCtrl(infoPanel, ID_EDT_NODEEDGEINFO, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE, wxDefaultValidator, wxT("edt_NodeEdgeInfo"));
		edt_NodeEdgeInfo->SetToolTip(wxT("Information about selected vertices and edges"));
		infoPanelSizer->Add(edt_NodeEdgeInfo,1 , wxGROW | wxALL, 5);

		stb_Legend = new wxStaticBox(infoPanel, ID_STB_LEGEND, wxT("Edge probabilities"), wxDefaultPosition, wxDefaultSize);
		wxStaticBoxSizer* StaticBoxSizer_Legend = new wxStaticBoxSizer(stb_Legend, wxVERTICAL);
		infoPanelSizer->Add(StaticBoxSizer_Legend, 0, wxGROW | wxALL, 5);
		
		GridSizer_Line = new wxGridSizer(11,2,0,10);
		StaticBoxSizer_Legend->Add(GridSizer_Line, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);

		stt_Line.clear();
		wnd_Line.clear();
		for (std::size_t i = 0; i <= 10; i++) {
			wnd_Line.push_back(new wxWindow(infoPanel, -1, wxDefaultPosition, wxSize(40, 5)));
			wnd_Line.at(i)->SetBackgroundColour(GraphCanvas::GetLineColor((double)i / 10));
			GridSizer_Line->Add(wnd_Line.at(i), 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
			wxString text;
			text << (i * 10) << wxT("%");
			stt_Line.push_back(new wxStaticText(infoPanel, -1, text, wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT));
			GridSizer_Line->Add(stt_Line.at(i), 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
		}
		ShowLegend(false);
	// Information Tab end
// OptInfo-Tabbing end

// Search Panel begin
	wxBoxSizer* boxsizer_Search = new wxBoxSizer(wxHORIZONTAL);
	boxsizer_Left->Add(boxsizer_Search, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

	btn_SearchPaths = new wxButton(wnd_Left, ID_BTN_SEARCHPATHS, wxT("Search"), wxDefaultPosition, wxSize(70,-1), 0, wxDefaultValidator, wxT("btn_SearchPaths"));
	btn_SearchPaths->SetToolTip(wxT("Starts and pauses the search"));
	boxsizer_Search->Add(btn_SearchPaths, 0, wxGROW | wxALL, 5);
	btn_SearchPaths->Disable();

	btn_StopSearch = new wxButton(wnd_Left, ID_BTN_STOPSEARCH, wxT("Stop"), wxDefaultPosition, wxSize(70,-1), 0, wxDefaultValidator, wxT("btn_StopSearch"));
	btn_StopSearch->SetToolTip(wxT("Stops the current running search"));
	boxsizer_Search->Add(btn_StopSearch, 0, wxGROW | wxALL, 5);
	btn_StopSearch->Disable();

	btn_DelResultTab = new wxButton(wnd_Left, ID_BTN_DELRESULTTAB, wxT("Del tab"), wxDefaultPosition, wxSize(70,-1), 0, wxDefaultValidator, wxT("btn_DelResultTab"));
	btn_DelResultTab->SetToolTip(wxT("Removes the currently selected result list tab"));
	btn_DelResultTab->Enable(false);
	boxsizer_Search->Add(btn_DelResultTab, 0, wxGROW | wxALL, 5);
// Search Panel end

// Graph, Resultlist Tabbing begin
	ntb_Graphs = new wxNotebook(wnd_Right, ID_NTB_GRAPHS, wxDefaultPosition, wxDefaultSize);
	boxsizer_Right->Add(ntb_Graphs, 1, wxGROW | wxALL, 5);

	ntb_Results = new wxNotebook(wnd_Bottom, ID_NTB_RESULTS, wxDefaultPosition, wxDefaultSize);
	boxsizer_Bottom->Add(ntb_Results, 1, wxGROW | wxALL, 5);
// Graph, Resultlist Tabbing end
}

void guiFrm::CreateToolBar(void) {
	ToolBar = new wxToolBar(this, ID_TOOLBAR, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL | wxNO_BORDER);
	ToolBar->SetToolBitmapSize(wxSize(24, 24));

   	wxBitmap bmpOpen(open_xpm);
	wxBitmap bmpSave(save_xpm);
	wxBitmap bmpSaveGraph(savegraph_xpm);
	wxBitmap bmpFind(find_xpm);
	wxBitmap bmpAdd(add_xpm);
	wxBitmap bmpDel(del_xpm);
	wxBitmap bmpZoom(zoom_xpm);
	wxBitmap bmpZoomIn(zoomin_xpm);
	wxBitmap bmpZoomOut(zoomout_xpm);

	ToolBar->AddTool(wxID_OPEN, wxT("Open"), bmpOpen, wxT("Open Result List..."));
	ToolBar->AddTool(wxID_SAVE, wxT("Save"), bmpSave, wxT("Save Result List as..."));
	ToolBar->AddSeparator();
	ToolBar->AddTool(ID_BTN_SAVEGRAPH, wxT("Save Graph"), bmpSaveGraph, wxT("Save Result Graph as..."));
	ToolBar->AddSeparator();
	ToolBar->AddTool(ID_BTN_ADDGRAPHTAB, wxT("Add tab"), bmpAdd, wxT("Add Graph Tab"));
	ToolBar->AddTool(ID_BTN_DELGRAPHTAB, wxT("Delete tab"), bmpDel, wxT("Delete Graph Tab"));
	ToolBar->AddSeparator();
	ToolBar->AddTool(ID_BTN_ZOOMFIT, wxT("Zoom fit"), bmpZoom, wxT("Zoom fit"));
	ToolBar->AddTool(ID_BTN_ZOOMIN, wxT("Zoom in"), bmpZoomIn, wxT("Zoom in"));
	ToolBar->AddTool(ID_BTN_ZOOMOUT, wxT("Zoom out"), bmpZoomOut, wxT("Zoom out"));

	wxArrayString cbx_Zoom_array;
	cbx_Zoom_array.Add(wxT("10%")); cbx_Zoom_array.Add(wxT("20%")); cbx_Zoom_array.Add(wxT("30%"));
	cbx_Zoom_array.Add(wxT("40%")); cbx_Zoom_array.Add(wxT("50%")); cbx_Zoom_array.Add(wxT("60%"));
	cbx_Zoom_array.Add(wxT("70%")); cbx_Zoom_array.Add(wxT("80%")); cbx_Zoom_array.Add(wxT("90%"));
	cbx_Zoom_array.Add(wxT("100%")); cbx_Zoom_array.Add(wxT("110%")); cbx_Zoom_array.Add(wxT("120%"));
	cbx_Zoom_array.Add(wxT("130%")); cbx_Zoom_array.Add(wxT("140%")); cbx_Zoom_array.Add(wxT("150%"));
	cbx_Zoom_array.Add(wxT("160%")); cbx_Zoom_array.Add(wxT("170%")); cbx_Zoom_array.Add(wxT("180%"));
	cbx_Zoom_array.Add(wxT("190%")); cbx_Zoom_array.Add(wxT("200%")); cbx_Zoom_array.Add(wxT("210%"));
	cbx_Zoom = new wxComboBox(ToolBar, ID_CBX_ZOOM, wxT("100%"), wxDefaultPosition, wxSize(70, -1), 
		cbx_Zoom_array, 0, wxDefaultValidator, wxT("cbx_Zoom"));
	cbx_Zoom->SetSelection(9);

	ToolBar->AddControl(cbx_Zoom);

	ToolBar->Realize();
	this->SetToolBar(ToolBar);
}

void guiFrm::CreateMenu(void) {
   	MenuBar = new wxMenuBar();

	wxMenu *mnu_File = new wxMenu(0);
	MenuBar->Append(mnu_File, wxT("&File"));
	mnu_File->Append(ID_MNU_LOADGRAPH, wxT("&Load &Network...\tCTRL+N"), wxT(""), wxITEM_NORMAL);
	mnu_File->AppendSeparator();	
	mnu_File->Append(wxID_OPEN, wxT("&Load Result List...\tCTRL+O"), wxT(""), wxITEM_NORMAL);
	mnu_File->Append(wxID_SAVE, wxT("&Save Result List as...\tCTRL+S"), wxT(""), wxITEM_NORMAL);
	mnu_File->AppendSeparator();	
	mnu_File->Append(ID_BTN_SAVEGRAPH, wxT("S&ave Result Graph as...\tCTRL+G"), wxT(""), wxITEM_NORMAL);
	mnu_File->AppendSeparator();	
	mnu_File->Append(wxID_EXIT, wxT("E&xit\tCTRL+Q"), wxT(""), wxITEM_NORMAL);
	
	wxMenu *mnu_Edit = new wxMenu(0);
	MenuBar->Append(mnu_Edit, wxT("&Edit"));
	mnu_Edit->Append(ID_MNU_OPTION, wxT("&Options..."), wxT(""), wxITEM_NORMAL);

	wxMenu *mnu_View = new wxMenu(0);
	MenuBar->Append(mnu_View, wxT("&View"));
	mnu_View->Append(ID_BTN_ZOOMIN, wxT("Zoom &In\tCTRL++"), wxT(""), wxITEM_NORMAL);
	mnu_View->Append(ID_BTN_ZOOMOUT, wxT("Zoom &Out\tCTRL+-"), wxT(""), wxITEM_NORMAL);
	mnu_View->Append(ID_BTN_ZOOMFIT, wxT("Zoom &Fit\tCTRL+F"), wxT(""), wxITEM_NORMAL);
	mnu_View->AppendSeparator();	
	mnu_View->Append(ID_MNU_REDRAW, wxT("&Redraw\tCTRL+R"), wxT(""), wxITEM_NORMAL);
	mnu_View->AppendSeparator();	
	mnu_View->Append(ID_MNU_SPLIT_LR, wxT("&Split Left/Right\tCTRL+L"), wxT(""), wxITEM_CHECK);
	mnu_View->Append(ID_MNU_SPLIT_TB, wxT("&Split Top/Bottom\tCTRL+T"), wxT(""), wxITEM_CHECK);
	mnu_View->Check(ID_MNU_SPLIT_LR, true);
	mnu_View->Check(ID_MNU_SPLIT_TB, true);

	wxMenu *mnu_Tabbing = new wxMenu(0);
	MenuBar->Append(mnu_Tabbing, wxT("&Tabbing"));
	mnu_Tabbing->Append(ID_BTN_ADDGRAPHTAB, wxT("&Add Graph Tab"), wxT(""), wxITEM_NORMAL);
	mnu_Tabbing->Append(ID_BTN_DELGRAPHTAB, wxT("&Delete Graph Tab"), wxT(""), wxITEM_NORMAL);
	mnu_Tabbing->AppendSeparator();
	mnu_Tabbing->Append(ID_MNU_DELRESULTTAB, wxT("Delete &Result List Tab"), wxT(""), wxITEM_NORMAL);

	wxMenu *mnu_Help = new wxMenu(0);
	MenuBar->Append(mnu_Help, wxT("&Help"));
	mnu_Help->Append(ID_MNU_HELPCONTENT, wxT("&Contents\tF1"), wxT(""), wxITEM_NORMAL);
	mnu_Help->AppendSeparator();
	mnu_Help->Append(ID_MNU_ABOUT, wxT("&About"), wxT(""), wxITEM_NORMAL);
	
	this->SetMenuBar(MenuBar);
}

void guiFrm::CreateStatusBar(void) {
	StatusBar = new wxStatusBar(this, ID_STATUSBAR);
	StatusBar->SetFieldsCount(2);
	StatusBar->SetStatusText(wxT("Welcome to FASPAD"),0);
	int StatusBar_Widths[2];
	StatusBar_Widths[0] = -2;
	StatusBar_Widths[1] = -1;
	StatusBar->SetStatusWidths(2,StatusBar_Widths);
	this->SetStatusBar(StatusBar);
}

void guiFrm::ManageWindow(void) {
	GetSizer()->Fit(this);
	GetSizer()->SetSizeHints(this);
	this->Center();
	this->SetIcon(wxNullIcon);
	SplitterWindow_TopBottom->SetSashPosition(-150);
	SplitterWindow_LeftRight->SetSashPosition(300);
}

void guiFrm::AddGraphTab(wxNotebook* ntb) {
	wxPanel *panel = new wxPanel(ntb);
	wxString name;
	name << wxT("Graph ") << (++graphTabNo);

	wxBoxSizer* panelSizer = new wxBoxSizer(wxHORIZONTAL);
	panel->SetSizer(panelSizer);
	panel->SetAutoLayout(TRUE);
	
	size_t size = Canvas.size();

    Canvas.push_back(GraphCanvas::CreateCanvas(NULL, panel));
	panelSizer->Add(Canvas[size], 1, wxGROW | wxALL, 0);
	
	Diagram.push_back(new GraphDiagram());
	Diagram[size]->SetCanvas(Canvas[size]);
	Canvas[size]->SetDiagram(Diagram[size]);
	ntb->AddPage(panel, name, true);
}

void guiFrm::AddResListTab(wxNotebook* ntb) {
	wxPanel *panel = new wxPanel(ntb);
	wxString name;
	name << wxT("Result list ") << (++resTabNo);

	wxBoxSizer* panelSizer = new wxBoxSizer(wxHORIZONTAL);
	panel->SetSizer(panelSizer);
	panel->SetAutoLayout(TRUE);
	
	size_t size = ResPathList.size();

    ResPathList.push_back(new ResultList(panel, wxID_ANY, wxDefaultPosition, wxDefaultSize));
	panelSizer->Add(ResPathList[size], 1, wxGROW | wxFIXED_MINSIZE | wxALL, 0);
	ntb->AddPage(panel, name, true);
	btn_DelResultTab->Enable(true);
}

void guiFrm::OnAddGraphTab(wxCommandEvent& WXUNUSED(event)) {
    AddGraphTab(ntb_Graphs);
}

void guiFrm::OnAddResListTab(wxCommandEvent& WXUNUSED(event)) {
    AddResListTab(ntb_Results);
    resListDone = true;
}

void read_vertex_file(const std::string& file, wxListBox* lbx) {
    std::ifstream in(file.c_str());
    if (!in) {
		wxString msg;	
		msg << wxT("Cannot open file ") << wxString(file.c_str(), wxConvUTF8) << wxT(".");
		wxMessageBox(msg, wxT("Error"), wxICON_WARNING);
		return;	    
    }
	std::vector<std::size_t> not_in;
	wxString errorMsg = wxT("The current loaded network does not contain the following node(s):\n\n");
    std::string line;
    std::size_t lineno = 0;
    std::set<vertex_t> vertices;
    while (std::getline(in, line)) {
	++lineno;
	std::string::size_type p = line.find('#');
	if (p != std::string::npos)
	    line = line.substr(0, p);
	line = trim(line);
	if (line.empty())
	    continue;
	if (line.find_first_of(WHITESPACE) != std::string::npos) {
		wxString msg;	
		msg << wxT("Syntax error in line ") << lineno << wxT(" while reading ") << wxString(file.c_str(), wxConvUTF8) << wxT(".");
		wxMessageBox(msg, wxT("Syntax error!"), wxICON_WARNING);
		return;	    
	}
	if (wxGetApp().g && (wxGetApp().g->lookup_vertex(line) == NULL)) {
		not_in.push_back(lbx->GetCount());
		if (not_in.size() != 1) errorMsg << wxT(", ");
		errorMsg << wxString(line.c_str(), wxConvUTF8);
	}
	lbx->Append(wxString(line.c_str(), wxConvUTF8));
	}
	if (not_in.size() > 0) {
		errorMsg << wxT("\n\nDo you really want to add it/them?");
		if(wxMessageBox(errorMsg, wxT("Error"), wxYES | wxNO | wxICON_WARNING, lbx) == wxNO) {
			for (int i = not_in.size() - 1; i >= 0; i--)
				lbx->Delete(not_in.at(i)); 
		}
	}
}

wxString wxFindAppPath(const wxString& argv0, const wxString& cwd, const wxString& appVariableName)
{
    wxString str;

    // Try appVariableName
    if (!appVariableName.IsEmpty())
    {
        str = wxGetenv(appVariableName);
        if (!str.IsEmpty())
            return str;
    }

#if defined(__WXMAC__) && !defined(__DARWIN__)
    // On Mac, the current directory is the relevant one when
    // the application starts.
    return cwd;
#endif

    if (wxIsAbsolutePath(argv0))
        return wxPathOnly(argv0);
    else
    {
        // Is it a relative path?
        wxString currentDir(cwd);
        if (currentDir.Last() != wxFILE_SEP_PATH)
            currentDir += wxFILE_SEP_PATH;

        str = currentDir + argv0;
        if (wxFileExists(str))
            return wxPathOnly(str);
    }

    // OK, it's neither an absolute path nor a relative path.
    // Search PATH.

    wxPathList pathList;
    pathList.AddEnvList(wxT("PATH"));
    str = pathList.FindAbsoluteValidPath(argv0);
    if (!str.IsEmpty())
        return wxPathOnly(str);

    // Failed
    return wxEmptyString;
}

void guiFrm::OnMenuExit(wxCommandEvent& WXUNUSED(event)) {
	loadNet->Destroy();
    Destroy();
}

void guiFrm::guiFrmClose(wxCloseEvent& WXUNUSED(event)) {
	if (search_status != 0) {
	    wxString msg;
	    msg << wxT("A search process is still running.\n")
			<< wxT("Do you want to stop it and quit FASPAD?");
	
	    wxMessageDialog msgDiag(this, msg, wxT("Information"), wxYES | wxNO | wxICON_INFORMATION);
		if (msgDiag.ShowModal() != wxID_YES) return;
		search_thread->Delete();
	}
	loadNet->Destroy();
	Destroy();
}

void guiFrm::OnBtnLoadGraphClick(wxCommandEvent& WXUNUSED(event)) {
    if (OpenFileDialog->ShowModal() == wxID_OK) {
		wxGetApp().graph_filename = OpenFileDialog->GetPath();
		wxGetApp().graphload_ok = false;
    	loadNet->ShowModal();
		if (!(wxGetApp().graphload_ok)) return;
		std::string graphfile = (std::string)OpenFileDialog->GetPath().mb_str(wxConvUTF8);
		std::ifstream graphin(graphfile.c_str());

		wxGetApp().graphload_ok = true;
    	Graph *g = new Graph(graphin, !(wxGetApp().modIsSPD));
		if (wxGetApp().graphload_ok) {
			if (g->num_vertices() == 0) {
				wxMessageBox(wxT("Error while loading graph!\nGraph is empty.\nLoading aborted."), wxT("Error"), wxICON_WARNING);
				delete g;
			} else {
				wxGetApp().g = g;
				edt_GraphFile->SetValue(OpenFileDialog->GetPath());
				btn_SearchPaths->Enable();
				EnableParameters(true);
			}
		} else {
			delete g;
		}
	}
}

void guiFrm::OnBtnSearchPathsClick(wxCommandEvent& WXUNUSED(event)) {     
	switch(search_status) {
		case 0: {
			ParamSet *parameter_set = new ParamSet;
			parameter_set->read_from_gui();
			if (!parameter_set->check()) return;

			btn_SearchPaths->SetLabel(wxT("Pause"));
			btn_StopSearch->Enable();
			//ResPathList[sel]->SetOld(-1);
			search_thread = new SearchThread(parameter_set);
			search_thread->Create();
			search_thread->Run();
			search_status = 1;
			break;
		}

		case 1: {
			search_thread->Pause();
			btn_SearchPaths->SetLabel(wxT("Resume"));
			search_status = 2;
			break;
		}

		case 2: {
			search_thread->Resume();
			btn_SearchPaths->SetLabel(wxT("Pause"));
			search_status = 1;
			break;
		}
	}			
}

void guiFrm::OnBtnStopSearchClick(wxCommandEvent& WXUNUSED(event)) {     
	search_thread->Delete();
	search_status = 0;
	btn_StopSearch->Disable();
	btn_SearchPaths->SetLabel(wxT("Search"));
	StatusBar->SetStatusText(wxT("Aborted"),1);
}

void guiFrm::OnBtnAddStartNodeClick(wxCommandEvent& WXUNUSED(event)) {
	if (edt_StartNode->GetValue().IsEmpty()) return;
	bool add_node = true;
	if (wxGetApp().g && 
			(wxGetApp().g->lookup_vertex((std::string)(edt_StartNode->GetValue().mb_str(wxConvUTF8))) == NULL)) {
		wxString msg = wxT("The current loaded network does not contain the node '");
		msg << edt_StartNode->GetValue() << wxT("'. ");
		msg << wxT("Do you really want to add it to the list?");
		if (wxMessageBox(msg, wxT("Error"), wxYES | wxNO | wxICON_WARNING, this) == wxNO)
			add_node = false;
	}
	if (add_node) {
    if (lbx_StartNodes->FindString(edt_StartNode->GetValue()) == wxNOT_FOUND) 
	   lbx_StartNodes->Append(edt_StartNode->GetValue());
    edt_StartNode->Clear();
	rbt_SomeStartNodes->SetValue(true);
	}
}

void guiFrm::OnBtnAddEndNodeClick(wxCommandEvent& WXUNUSED(event)) {
	if (edt_EndNode->GetValue().IsEmpty()) return;
	bool add_node = true;
	if (wxGetApp().g && 
			(wxGetApp().g->lookup_vertex((std::string)(edt_EndNode->GetValue().mb_str(wxConvUTF8))) == NULL)) {
		wxString msg = wxT("The current loaded network does not contain the node '");
		msg << edt_EndNode->GetValue() << wxT("'. ");
		msg << wxT("Do you really want to add it to the list?");
		if (wxMessageBox(msg, wxT("Error"), wxYES | wxNO | wxICON_WARNING, this) == wxNO)
			add_node = false;
	}
	if (add_node) {
    if (lbx_EndNodes->FindString(edt_EndNode->GetValue()) == wxNOT_FOUND)
	   lbx_EndNodes->Append(edt_EndNode->GetValue());
    edt_EndNode->Clear();
	rbt_SomeEndNodes->SetValue(true);
	}
}

void guiFrm::OnBtnLoadStartNodeFileClick(wxCommandEvent& WXUNUSED(event)) {
	if (OpenFileDialog->ShowModal() == wxID_OK) {
		std::string vertex_file = (std::string)OpenFileDialog->GetPath().mb_str(wxConvUTF8);
		lbx_StartNodes->Clear();
		read_vertex_file(vertex_file, lbx_StartNodes);
		rbt_SomeStartNodes->SetValue(true);
	}
}

void guiFrm::OnBtnLoadEndNodeFileClick(wxCommandEvent& WXUNUSED(event)) {
	if (OpenFileDialog->ShowModal() == wxID_OK) {
		std::string vertex_file = (std::string)OpenFileDialog->GetPath().mb_str(wxConvUTF8);
		lbx_EndNodes->Clear();
		read_vertex_file(vertex_file, lbx_EndNodes);
		rbt_SomeEndNodes->SetValue(true);
	}
}

void guiFrm::OnBtnDelStartNodeClick(wxCommandEvent& WXUNUSED(event)) {
	wxArrayInt selections;
	lbx_StartNodes->GetSelections(selections);
	for (int i = selections.GetCount() - 1; i >= 0; i--) {
       lbx_StartNodes->Delete(selections[i]);
    }
}

void guiFrm::OnBtnDelEndNodeClick(wxCommandEvent& WXUNUSED(event)) {
	wxArrayInt selections;
	lbx_EndNodes->GetSelections(selections);
	for (int i = selections.GetCount() - 1; i >= 0; i--) {
       lbx_EndNodes->Delete(selections[i]);
    }
}

void guiFrm::OnBtnZoomFitClick(wxCommandEvent& WXUNUSED(event)) {
	int graph_sel = ntb_Graphs->GetSelection();
	if (graph_sel < 0) return;
	Canvas[graph_sel]->ZoomFit();
}

void guiFrm::OnBtnZoomInClick(wxCommandEvent& WXUNUSED(event)) {
	int sel;
	if (cbx_Zoom->GetStringSelection().Cmp(cbx_Zoom->GetValue()) == 0)
		sel = cbx_Zoom->GetSelection();
	else
		sel = (int)(wxAtof(cbx_Zoom->GetValue()) / 10) - 1;
	if (sel == cbx_Zoom->GetCount() - 1) return;

	cbx_Zoom->SetSelection(sel + 1);
	wxCommandEvent changeevent(wxEVT_COMMAND_COMBOBOX_SELECTED, ID_CBX_ZOOM);
	changeevent.SetEventObject(cbx_Zoom);
	GetEventHandler()->ProcessEvent(changeevent);
}

void guiFrm::OnBtnZoomOutClick(wxCommandEvent& WXUNUSED(event)) {
	int sel;
	if (cbx_Zoom->GetStringSelection().Cmp(cbx_Zoom->GetValue()) == 0)
		sel = cbx_Zoom->GetSelection();
	else
		sel = (int)(wxAtof(cbx_Zoom->GetValue()) / 10);
	if (sel <= 0) return;

	cbx_Zoom->SetSelection(sel - 1);
	wxCommandEvent changeevent(wxEVT_COMMAND_COMBOBOX_SELECTED, ID_CBX_ZOOM);
	changeevent.SetEventObject(cbx_Zoom);
	GetEventHandler()->ProcessEvent(changeevent);
}

void guiFrm::OnCbxZoomChange(wxCommandEvent& event) {
	wxComboBox* cbx = (wxComboBox*) event.GetEventObject();
	wxASSERT(cbx != NULL);
	int sel = ntb_Graphs->GetSelection();
	if (sel == -1) return;

	wxClientDC dc(Canvas[sel]);
	Canvas[sel]->Zoom((cbx->GetSelection() + 1) * 0.1);
	Canvas[sel]->Refresh(); 
}

void guiFrm::OnBtnAddGraphTabClick(wxCommandEvent& WXUNUSED(event)) {
	AddGraphTab(ntb_Graphs);
}

void guiFrm::OnBtnDelGraphTabClick(wxCommandEvent& WXUNUSED(event)) {
	int sel = ntb_Graphs->GetSelection();
	if (sel == -1) return;
	
	std::vector<GraphDiagram*>::iterator diter = Diagram.begin();
	for(int i = 0; i < sel; i++) diter++;
	delete *diter;
	Diagram.erase(diter);

	std::vector<GraphCanvas*>::iterator citer = Canvas.begin();
	for(int i = 0; i < sel; i++) citer++;
	delete *citer;
	Canvas.erase(citer);

	ntb_Graphs->DeletePage(sel);
}

void guiFrm::OnBtnAddResultTabClick(wxCommandEvent& WXUNUSED(event)) {
	AddResListTab(ntb_Results);
}

void guiFrm::OnBtnDelResultTabClick(wxCommandEvent& WXUNUSED(event)) {
	int sel = ntb_Results->GetSelection();
	if (sel == -1) return;
	
	std::vector<ResultList*>::iterator iter = ResPathList.begin();
	for(int i = 0; i < sel; i++) iter++;
	delete *iter;
	ResPathList.erase(iter);

	ntb_Results->DeletePage(sel);
	if (ntb_Results->GetPageCount() == 0) btn_DelResultTab->Enable(false);
}

void guiFrm::OnNtbGraphsTabChanged(wxNotebookEvent& event) {
	int graph_sel = event.GetSelection();
	int result_sel = ntb_Results->GetSelection();
	if ((graph_sel < 0) || (result_sel < 0)) return;
	if ((graph_sel >= Diagram.size()) || (result_sel >= ResPathList.size())) return;

	if (Diagram[graph_sel]->GetCount() > 0) ShowLegend(true);
	else ShowLegend(false);
	ResPathList[result_sel]->UpdateParam(Diagram[graph_sel]->GetDiagramContent());
	wxString scale;
	wxString format = wxT("%6.2f");
	scale.Printf(format.c_str(), (Diagram[graph_sel]->g_scale * 100));
	scale << wxT("%");
	cbx_Zoom->SetValue(scale);
}

void guiFrm::OnNtbResultsTabChanged(wxNotebookEvent& event) {
	int graph_sel = ntb_Graphs->GetSelection();
	int result_sel = event.GetSelection();
	if ((graph_sel < 0) || (result_sel < 0)) return;
	if ((graph_sel >= Diagram.size()) || (result_sel >= ResPathList.size())) return;

	ResPathList[result_sel]->UpdateParam(Diagram[graph_sel]->GetDiagramContent());
	wxString scale;
	wxString format = wxT("%6.2f");
	scale.Printf(format.c_str(), (Diagram[graph_sel]->g_scale * 100));
	scale << wxT("%");
	cbx_Zoom->SetValue(scale);
}

void guiFrm::OnBtnSaveResultsClick(wxCommandEvent& WXUNUSED(event)) {
	int result_sel = ntb_Results->GetSelection();
	if (result_sel < 0) return;
	ResPathList[result_sel]->SaveList();
}

void guiFrm::OnBtnLoadResultsClick(wxCommandEvent& WXUNUSED(event)) {
	AddResListTab(ntb_Results);
	int result_sel = ntb_Results->GetSelection();
	if (result_sel < 0) return;
	if(!ResPathList[result_sel]->LoadList()) {
		std::vector<ResultList*>::iterator iter = ResPathList.begin();
		for(int i = 0; i < result_sel; i++) iter++;
		delete *iter;
		ResPathList.erase(iter);
		ntb_Results->DeletePage(result_sel);
		if (ntb_Results->GetPageCount() == 0) btn_DelResultTab->Enable(false);
	}
}

void guiFrm::OnBtnExportGraphClick(wxCommandEvent& WXUNUSED(event)) {
	int graph_sel = ntb_Graphs->GetSelection();
	if (graph_sel < 0) return;
	Canvas[graph_sel]->ExportGraph();
}

void guiFrm::OnBtnPrintGraphClick(wxCommandEvent& WXUNUSED(event)) {
	int graph_sel = ntb_Graphs->GetSelection();
	if (graph_sel < 0) return;
	Canvas[graph_sel]->PrintGraph();
}

void guiFrm::OnMenuOptions(wxCommandEvent& WXUNUSED(event)) {
    optionFrm *opt = new optionFrm(NULL);
    opt->ShowModal();		
}

void guiFrm::OnMenuRedraw(wxCommandEvent& WXUNUSED(event)) {
	int graph_sel = ntb_Graphs->GetSelection();
	if (graph_sel < 0) return;
	Canvas[graph_sel]->Zoom(Diagram[graph_sel]->g_scale);
}

void guiFrm::OnMenuSplitLR(wxCommandEvent& WXUNUSED(event)) {
	if (SplitterWindow_LeftRight->IsSplit()) {
		sashPosLR = SplitterWindow_LeftRight->GetSashPosition();
		SplitterWindow_LeftRight->Unsplit(wnd_Left);
	}
	else SplitterWindow_LeftRight->SplitVertically(wnd_Left, wnd_Right, sashPosLR);
}

void guiFrm::OnMenuSplitTB(wxCommandEvent& WXUNUSED(event)) {
	if (SplitterWindow_TopBottom->IsSplit()) {
		sashPosTB = SplitterWindow_TopBottom->GetSashPosition();
		SplitterWindow_TopBottom->Unsplit(wnd_Bottom);
	}
	else SplitterWindow_TopBottom->SplitHorizontally(SplitterWindow_LeftRight, wnd_Bottom, sashPosTB);
}

void guiFrm::OnHelpContent(wxCommandEvent& WXUNUSED(event)) {
	wxString file = wxFindAppPath(wxTheApp->argv[0], wxGetCwd(), wxT(""));
#ifdef __WXMSW__
	file << wxT("\\");
#else
	file << wxT("/");
#endif
	file << wxT("faspad-manual.pdf");
	bool ok = false;
	wxFileType *ft = wxTheMimeTypesManager->GetFileTypeFromExtension(wxT("pdf"));

	if ( ft ) {
	    wxString cmd;
	    ok = ft->GetOpenCommand(&cmd, wxFileType::MessageParameters(file, wxEmptyString));
	    delete ft;
	    if (ok) ok = (wxExecute(cmd, wxEXEC_ASYNC) != 0);
	}
	if (!ok) {
		wxString msg = wxT("An error occured while opening the manual.\n");
		msg << wxT("Try to open the file faspad-manual.pdf in a pdf-viewer.");
		wxMessageBox(msg, wxT("Error"), wxOK | wxICON_WARNING, this);
	}
}

void guiFrm::OnAbout(wxCommandEvent& WXUNUSED(event)) {
    wxString msg;
    msg << wxT("Fast Signaling Pathway Detection 1.0\nApril 15, 2007\n\nFalk Hueffner, Sebastian Wernicke, Thomas Zichner\n")
		<< wxT("{hueffner,wernicke,tzi}@minet.uni-jena.de");

    wxMessageBox(msg, wxT("About FASPAD"), wxOK | wxICON_INFORMATION, this);
}

void guiFrm::ShowLegend(bool show) {
	for (std::size_t i = 0; i <= 10; i++)
		stt_Line.at(i)->Show(show);

	if (!show) return;

	int graph_sel = ntb_Graphs->GetSelection();
	if (graph_sel < 0) return;
	GraphDiagram *gdiag = Diagram[graph_sel];

	wxString scale;
	float start, interval;

	if (gdiag->modIsSPD) {
		scale = wxT("%");
		start = 0;
		interval = 10;
	} else {
		scale = wxT("");
		start = gdiag->weightMax;
		interval = - (gdiag->weightDiff) / 10;
	}

	wxString label;
	wxString format = wxT("%6.2f");
	for (std::size_t i = 0; i <= 10; i++) {
		label.Clear();
		label.Printf(format.c_str(), (i * interval + start));
		label << scale;
		stt_Line.at(i)->SetLabel(label);
	}
	infoPanelSizer->Layout();
}

void guiFrm::EnableParameters(bool value) {
	edt_Pathlength->Enable(value);
	edt_NumPaths->Enable(value);
	edt_Filter->Enable(value);
	edt_SuccessProb->Enable(value);
	rbt_AllStartNodes->Enable(value);
	rbt_SomeStartNodes->Enable(value);
	btn_AddStartNode->Enable(value);
	edt_StartNode->Enable(value);
	btn_DelStartNode->Enable(value);
	btn_LoadStartNodeFile->Enable(value);
	rbt_AllEndNodes->Enable(value);
	rbt_SomeEndNodes->Enable(value);
	btn_AddEndNode->Enable(value);
	edt_EndNode->Enable(value);
	btn_DelEndNode->Enable(value);
	btn_LoadEndNodeFile->Enable(value);

}
