#ifndef GUIFRM_H
#define GUIFRM_H

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

#ifndef WX_PRECOMP
	#include <wx/wx.h>
	#include <wx/frame.h>
#else
	#include <wx/wxprec.h>
#endif

#include <wx/docview.h>
#include <wx/notebook.h>
#include <wx/menu.h>
#include <wx/filedlg.h>
#include <wx/statusbr.h>
#include <wx/toolbar.h>
#include <wx/panel.h>
#include <wx/listbox.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/scrolwin.h>
#include <wx/splitter.h>
#include <wx/ogl/ogl.h>

#include "loadNetFrm.h"
#include "searchthread.h"
#include "resultlist.h"
#include "graphcanvas.h"

////Dialog Style Start
#undef guiFrm_STYLE
#define guiFrm_STYLE wxCAPTION | wxRESIZE_BORDER | wxSYSTEM_MENU | wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxCLOSE_BOX
////Dialog Style End

class guiFrm : public wxFrame
{
	private:
		std::size_t graphTabNo;
		std::size_t resTabNo;
		int sashPosLR;
		int sashPosTB;
		SearchThread *search_thread;
		loadNetFrm *loadNet;
		DECLARE_EVENT_TABLE();
		
	public:
		guiFrm(wxWindow *parent, wxWindowID id = 1, const wxString &title = wxT("Fast Signaling Pathway Detection"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = guiFrm_STYLE);
		
	public:
        std::vector<GraphCanvas*> Canvas;
        std::vector<GraphDiagram*> Diagram;
		std::vector<ResultList*> ResPathList;
		std::size_t search_status;
		bool resListDone;
		wxFileDialog *OpenFileDialog;

		wxMenuBar *MenuBar;
		wxToolBar *ToolBar;
		wxStatusBar *StatusBar;

		wxSplitterWindow *SplitterWindow_TopBottom;
		wxSplitterWindow *SplitterWindow_LeftRight;
		wxScrolledWindow *wnd_Left;
		wxScrolledWindow *wnd_Right;
		wxScrolledWindow *wnd_Bottom;

		wxButton *btn_AddGraphTab;
		wxButton *btn_DelGraphTab;
		wxButton *btn_AddResultTab;
		wxButton *btn_DelResultTab;
		wxComboBox *cbx_Zoom;

		wxNotebook *ntb_OptInfo;
		wxNotebook *ntb_Options;
		wxBoxSizer* optPanelMainSizer;
		wxButton *btn_LoadGraph;
		wxTextCtrl *edt_GraphFile;
		wxTextCtrl *edt_Pathlength;
		wxStaticText *stt_Pathlength;
		wxTextCtrl *edt_NumPaths;
		wxStaticText *stt_NumPaths;
		wxTextCtrl *edt_Filter;
		wxStaticText *stt_Filter;
		wxTextCtrl *edt_SuccessProb;
		wxStaticText *stt_SuccessProb;

		wxRadioButton *rbt_AllStartNodes;
		wxRadioButton *rbt_SomeStartNodes;
		wxListBox *lbx_StartNodes;
		wxButton *btn_AddStartNode;
		wxTextCtrl *edt_StartNode;
		wxButton *btn_DelStartNode;
		wxButton *btn_LoadStartNodeFile;

		wxRadioButton *rbt_AllEndNodes;
		wxRadioButton *rbt_SomeEndNodes;
		wxListBox *lbx_EndNodes;
		wxButton *btn_AddEndNode;
		wxTextCtrl *edt_EndNode;
		wxButton *btn_DelEndNode;
		wxButton *btn_LoadEndNodeFile;

		wxTextCtrl *edt_NodeEdgeInfo;
		wxBoxSizer* infoPanelSizer;
		wxGridSizer* GridSizer_Line;
		wxStaticBox *stb_Legend;
		std::vector<wxStaticText *> stt_Line;
		std::vector<wxWindow *> wnd_Line;

		wxButton *btn_SearchPaths;
		wxButton *btn_StopSearch;

		wxNotebook *ntb_Graphs;
		wxNotebook *ntb_Results;

	public:
		enum
		{
			ID_MNU_FILE = 101,
			ID_MNU_LOADGRAPH = 102,
			ID_MNU_EXIT = 103,
			ID_MNU_EDIT = 104,
			ID_MNU_OPTION = 105,
			ID_MNU_VIEW = 106,
			ID_MNU_REDRAW = 107,
			ID_MNU_SPLIT_LR = 108,
			ID_MNU_SPLIT_TB = 109,
			ID_MNU_DELRESULTTAB = 110,
			ID_MNU_HELP = 111,
			ID_MNU_HELPCONTENT = 112,
			ID_MNU_ABOUT = 113,
			
			ID_STATUSBAR = 11,
			ID_TOOLBAR = 12,

			ID_SPLITTERWINDOW_TOPBOTTOM = 201,
			ID_SPLITTERWINDOW_LEFTRIGHT = 202,
			ID_WND_LEFT = 203,
			ID_WND_RIGHT = 204,
			ID_WND_BOTTOM = 205,

			ID_BTN_SAVEGRAPH = 301,
			ID_BTN_ADDGRAPHTAB = 302,
			ID_BTN_DELGRAPHTAB = 303,
			ID_BTN_ADDRESULTTAB = 304,
			ID_BTN_DELRESULTTAB = 305,
			ID_BTN_ZOOMFIT = 306,
			ID_BTN_ZOOMIN = 307,
			ID_BTN_ZOOMOUT = 308,
			ID_CBX_ZOOM = 309,

			ID_NTB_OPTINFO = 1001,
			ID_NTB_OPTIONS = 1002,
			ID_EDT_GRAPHFILE = 1003,
			ID_BTN_LOADGRAPH = 1004,
			ID_EDT_PATHLENGTH = 1005,
			ID_STT_PATHLENGTH = 1006,
			ID_EDT_NUMPATHS = 1007,
			ID_STT_NUMPATHS = 1008,
			ID_EDT_FILTER = 1009,
			ID_STT_FILTER = 1010,
			ID_EDT_SUCCESSPROB = 1011,
			ID_STT_SUCCESSPROB = 1012,
			ID_RBT_ALLSTARTNODES = 1013,
			ID_RBT_SOMESTARTNODES = 1014,
			ID_LBX_STARTNODES = 1015,
			ID_BTN_ADDSTARTNODE = 1016,
			ID_EDT_STARTNODE = 1017,
			ID_BTN_DELSTARTNODE = 1018,
			ID_BTN_LOADSTARTNODEFILE = 1019,
			ID_RBT_ALLENDNODES = 1020,
			ID_RBT_SOMEENDNODES = 1021,
			ID_LBX_ENDNODES = 1022,
			ID_BTN_ADDENDNODE = 1023,
			ID_EDT_ENDNODE = 1024,
			ID_BTN_DELENDNODE = 1025,
			ID_BTN_LOADENDNODEFILE = 1026,
			ID_EDT_NODEEDGEINFO = 1027,
			ID_STB_LEGEND = 1028,

			ID_BTN_SEARCHPATHS = 1029,
			ID_BTN_STOPSEARCH = 1030,
			ID_NTB_GRAPHS = 1031,
			ID_NTB_RESULTS = 1032,
		};
		
	public:
		void CreateControls(void);
		void CreateToolBar(void);
		void CreateMenu(void);
		void CreateStatusBar(void);
		void ManageWindow(void);
		void AddGraphTab(wxNotebook* ntb);
		void AddResListTab(wxNotebook* ntb);
		void OnAddGraphTab(wxCommandEvent& event);
		void OnAddResListTab(wxCommandEvent& event);
		void OnMenuExit(wxCommandEvent& event);
		void guiFrmClose(wxCloseEvent& event);
		void OnBtnLoadGraphClick(wxCommandEvent& event);
		void OnBtnSearchPathsClick(wxCommandEvent& event);
		void OnBtnStopSearchClick(wxCommandEvent& event);
		void OnBtnAddStartNodeClick(wxCommandEvent& event);
		void OnBtnAddEndNodeClick(wxCommandEvent& event);
		void OnBtnLoadStartNodeFileClick(wxCommandEvent& event);
		void OnBtnLoadEndNodeFileClick(wxCommandEvent& event);
		void OnBtnDelStartNodeClick(wxCommandEvent& event);
		void OnBtnDelEndNodeClick(wxCommandEvent& event);
		void OnBtnZoomFitClick(wxCommandEvent& event);
		void OnBtnZoomInClick(wxCommandEvent& event);
		void OnBtnZoomOutClick(wxCommandEvent& event);
		void OnCbxZoomChange(wxCommandEvent& event);
		void OnBtnAddGraphTabClick(wxCommandEvent& event);
		void OnBtnDelGraphTabClick(wxCommandEvent& event);
		void OnBtnAddResultTabClick(wxCommandEvent& event);
		void OnBtnDelResultTabClick(wxCommandEvent& event);
		void OnNtbGraphsTabChanged(wxNotebookEvent& event);
		void OnNtbResultsTabChanged(wxNotebookEvent& event);
		void OnBtnSaveResultsClick(wxCommandEvent& event);
		void OnBtnLoadResultsClick(wxCommandEvent& event);
		void OnBtnExportGraphClick(wxCommandEvent& event);
		void OnBtnPrintGraphClick(wxCommandEvent& event);
		void OnMenuOptions(wxCommandEvent& event);
		void OnMenuRedraw(wxCommandEvent& event);
		void OnMenuSplitLR(wxCommandEvent& event);
		void OnMenuSplitTB(wxCommandEvent& event);
		void OnHelpContent(wxCommandEvent& event);
		void OnAbout(wxCommandEvent& event);
		void ShowLegend(bool show = true);
		void EnableParameters(bool value = true);
};

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(EVT_ADD_GRAPH_TAB, -2)
DECLARE_EVENT_TYPE(EVT_ADD_RESLIST_TAB, -3)
END_DECLARE_EVENT_TYPES()

#endif // GUIFRM_H
