#include <wx/config.h>
#include <wx/colordlg.h>

#include "guiApp.h"
#include "optionFrm.h"

BEGIN_EVENT_TABLE(optionFrm,wxDialog)
	EVT_BUTTON(wxID_OK,optionFrm::OnBtnOKClick)
	EVT_BUTTON(wxID_CANCEL,optionFrm::OnBtnCancelClick)
	EVT_BUTTON(ID_BTN_COLOR1,optionFrm::OnBtnColorClick)
	EVT_BUTTON(ID_BTN_COLOR2,optionFrm::OnBtnColorClick)
END_EVENT_TABLE()

optionFrm::optionFrm(wxWindow *parent, wxWindowID id, const wxString &title, const wxPoint &position, const wxSize& size, long style)
: wxDialog(parent, id, title, position, size, style) {
	CreateOptionControls();
}

void optionFrm::CreateOptionControls(void) {
	wxBoxSizer* BoxSizer1 = new wxBoxSizer(wxVERTICAL);
	this->SetSizer(BoxSizer1);
	this->SetAutoLayout(TRUE);

	wxArrayString options_rbx_GraphType;
	options_rbx_GraphType.Add(wxT("Signaling Pathway Detection"));
	options_rbx_GraphType.Add(wxT("Minimum Path Search"));
	rbx_GraphType = new wxRadioBox(this, ID_RBX_GRAPHTYPE, wxT("Software usage"), wxDefaultPosition, wxDefaultSize, options_rbx_GraphType);
	BoxSizer1->Add(rbx_GraphType, 0, wxGROW | wxALL, 5);
	
	stb_Colors = new wxStaticBox(this, ID_STB_COLORS, wxT("Colors"), wxDefaultPosition, wxDefaultSize);
	wxStaticBoxSizer* StaticBoxSizer_Colors = new wxStaticBoxSizer(stb_Colors, wxVERTICAL);

	BoxSizer1->Add(StaticBoxSizer_Colors, 0, wxGROW | wxALL, 5);
	
	wxFlexGridSizer* GridSizer_Colors = new wxFlexGridSizer(2, 2, 0, 30);
	StaticBoxSizer_Colors->Add(GridSizer_Colors, 0, wxALL, 5);
		
	wxStaticText *stt_Color1 = new wxStaticText(this, -1, wxT("Vertex color for selected path"), wxDefaultPosition, wxDefaultSize);
	GridSizer_Colors->Add(stt_Color1, 0, wxGROW | wxALL, 2);
	btn_Color1 = new wxButton(this, ID_BTN_COLOR1, wxT(""), wxDefaultPosition, wxSize(50, 20));
	GridSizer_Colors->Add(btn_Color1, 0,wxALIGN_CENTER_VERTICAL | wxALL, 2);

	wxStaticText *stt_Color2 = new wxStaticText(this, -1, wxT("Color for the other vertices"), wxDefaultPosition, wxDefaultSize);
	GridSizer_Colors->Add(stt_Color2, 0, wxGROW | wxALL, 2);
	btn_Color2 = new wxButton(this, ID_BTN_COLOR2, wxT(""), wxDefaultPosition, wxSize(50, 20));
	GridSizer_Colors->Add(btn_Color2, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);


	stb_Misc = new wxStaticBox(this, ID_STB_MISC, wxT("Miscellaneous"), wxDefaultPosition, wxDefaultSize);
	wxStaticBoxSizer* StaticBoxSizer_Misc = new wxStaticBoxSizer(stb_Misc, wxVERTICAL);

	BoxSizer1->Add(StaticBoxSizer_Misc, 0, wxGROW | wxALL, 5);
	
	wxGridSizer* GridSizer_Misc = new wxGridSizer(2, 2, 0, 30);
	StaticBoxSizer_Misc->Add(GridSizer_Misc, 1, wxGROW | wxALL, 5);
		
	wxStaticText *stt_LineWidth = new wxStaticText(this, -1, wxT("Line width"), wxDefaultPosition, wxDefaultSize);
	GridSizer_Misc->Add(stt_LineWidth, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
	edt_LineWidth = new wxTextCtrl(this, ID_EDT_LINEWIDTH, wxT(""), wxDefaultPosition, wxSize(50, -1));
	GridSizer_Misc->Add(edt_LineWidth, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 2);

	wxStaticText *stt_FontSize = new wxStaticText(this, -1, wxT("Font size"), wxDefaultPosition, wxDefaultSize);
	GridSizer_Misc->Add(stt_FontSize, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
	edt_FontSize = new wxTextCtrl(this, ID_EDT_FONTSIZE, wxT(""), wxDefaultPosition, wxSize(50, -1));
	GridSizer_Misc->Add(edt_FontSize, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 2);

	siz_OkCancel = this->CreateButtonSizer(wxOK | wxCANCEL);
	BoxSizer1->Add(siz_OkCancel, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

	this->Fit();

	rbx_GraphType->SetSelection((wxGetApp().modIsSPD) ? 0 : 1);	
	btn_Color1->SetBackgroundColour(wxGetApp().colorSelV);
	btn_Color2->SetBackgroundColour(wxGetApp().colorDeSelV);
	wxString number;
	number << wxGetApp().lineWidth;
	edt_LineWidth->SetValue(number);
	number.Clear();
	number << wxGetApp().fontSize;
	edt_FontSize->SetValue(number);
}

void optionFrm::optionFrmClose(wxCloseEvent& WXUNUSED(event)) {
    Destroy();
}

void optionFrm::OnBtnOKClick(wxCommandEvent& WXUNUSED(event)) {
	if ((rbx_GraphType->GetSelection() == 0) && !wxGetApp().modIsSPD) {
		wxGetApp().modIsSPD = true;
		wxGetApp().frame->btn_LoadGraph->SetLabel(wxT("Load Network"));
		wxGetApp().frame->btn_LoadGraph->SetToolTip(wxT("Loads the network file to be searched in"));
		wxGetApp().frame->edt_GraphFile->Clear();
		wxGetApp().frame->btn_SearchPaths->Enable(false);
		wxGetApp().frame->EnableParameters(false);
		wxGetApp().frame->optPanelMainSizer->Layout();
	} else if ((rbx_GraphType->GetSelection() == 1) && wxGetApp().modIsSPD) {
		wxGetApp().modIsSPD = false;
		wxGetApp().frame->btn_LoadGraph->SetLabel(wxT("Load Graph"));
		wxGetApp().frame->btn_LoadGraph->SetToolTip(wxT("Loads the graph file to be searched in"));
		wxGetApp().frame->edt_GraphFile->Clear();
		wxGetApp().frame->btn_SearchPaths->Enable(false);
		wxGetApp().frame->EnableParameters(false);
		wxGetApp().frame->optPanelMainSizer->Layout();
	}
	wxConfig config(wxT("FASPAD"), wxT("FASPAD"));
	config.Write(wxT("General/modIsSPD"), wxGetApp().modIsSPD);

	wxGetApp().colorSelV = btn_Color1->GetBackgroundColour();
	wxGetApp().colorDeSelV = btn_Color2->GetBackgroundColour();

	config.Write(wxT("General/colorSelV_red"), wxGetApp().colorSelV.Red());
	config.Write(wxT("General/colorSelV_green"),wxGetApp().colorSelV.Green());
	config.Write(wxT("General/colorSelV_blue"), wxGetApp().colorSelV.Blue());

	config.Write(wxT("General/colorDeSelV_red"), wxGetApp().colorDeSelV.Red());
	config.Write(wxT("General/colorDeSelV_green"),wxGetApp().colorDeSelV.Green());
	config.Write(wxT("General/colorDeSelV_blue"), wxGetApp().colorDeSelV.Blue());

	wxGetApp().lineWidth = wxAtoi(edt_LineWidth->GetValue());
	wxGetApp().fontSize = wxAtoi(edt_FontSize->GetValue());
	config.Write(wxT("General/lineWidth"), wxGetApp().lineWidth);
	config.Write(wxT("General/fontSize"), wxGetApp().fontSize);

	Destroy();
}

void optionFrm::OnBtnCancelClick(wxCommandEvent& WXUNUSED(event)) {
	Destroy();
}

void optionFrm::OnBtnColorClick(wxCommandEvent& event) {
	wxButton* btn_Color = (wxButton*) event.GetEventObject();

	wxColourData colorData;
	colorData.SetColour(btn_Color->GetBackgroundColour());
	wxColourDialog colorDlg(this, &colorData);
	if (colorDlg.ShowModal() == wxID_OK) {
		btn_Color->SetBackgroundColour(colorDlg.GetColourData().GetColour());
	}
}
