#include "guiApp.h"
#include "paramset.h"

ParamSet::ParamSet() {
	m_path_length = 8;
	m_num_colors = 0;
	m_filter = 70;
	m_num_paths = 50;
    m_num_trials = 0;
    m_success_prob = 99.9;
    m_preheat_trials = 50;
    m_mode = Bounds::DYNPROG;
    m_max_lb_edges = 2;
}

void ParamSet::read_from_gui() {
	if (!wxGetApp().frame->edt_Pathlength->GetValue().IsEmpty()) 
		m_path_length = wxAtoi(wxGetApp().frame->edt_Pathlength->GetValue());
    if (!wxGetApp().frame->edt_NumPaths->GetValue().IsEmpty()) 
		m_num_paths = wxAtoi(wxGetApp().frame->edt_NumPaths->GetValue());
    if (!wxGetApp().frame->edt_Filter->GetValue().IsEmpty()) 
		m_filter = wxAtof(wxGetApp().frame->edt_Filter->GetValue());
    if (!wxGetApp().frame->edt_SuccessProb->GetValue().IsEmpty()) 
		m_success_prob = wxAtof(wxGetApp().frame->edt_SuccessProb->GetValue());
    if (!wxGetApp().frame->edt_GraphFile->GetValue().IsEmpty()) 
		m_graphfile = wxGetApp().frame->edt_GraphFile->GetValue();
}

bool ParamSet::check() {
	wxString errors = _T("The following error(s) occurred:\n");
	bool err_occurred = false;

	if (m_graphfile.IsEmpty()) {
	errors << _T("No graphfile specified.\n");
	err_occurred = true;
    }

    if (wxGetApp().g->num_vertices() > MAX_VERTEX) {
	errors << _T("Graph has") << wxGetApp().g->num_vertices() << _T(" vertices, but only")
		  << MAX_VERTEX << _T(" supported.\n");
	err_occurred = true;
    }

	if (m_path_length < 2) {
	errors << _T("Path length must be >= 2.\n");
	err_occurred = true;
    }

	if (m_path_length > (int)MAX_COLORS) {
	errors << _T("Path length must be <= ") << MAX_COLORS << _T(".\n");
	err_occurred = true;
    }
    if (m_num_colors == 0)
	m_num_colors = m_path_length;

//	if (m_num_colors < m_path_length) {
//	errors << _T("Need at least as many colors as the path length.\n");
//	err_occurred = true;
//	}

//	if (m_num_colors > MAX_COLORS) {
//	errors << _T("Number of colors must be <= ") << MAX_COLORS << _T(".\n");
//	err_occurred = true;
//	}

    if (m_num_paths < 1) {
	errors << _T("Number of result-paths must be >= 1.\n");
	err_occurred = true;
    }

    if ((m_filter < 0) || (m_filter > 100)) {
	errors << _T("Filter value must be between 0 and 100.\n");
	err_occurred = true;
    }

    if ((m_success_prob < 0) || (m_success_prob > 100)) {
	errors << _T("Success probability must be between 0 and 100.\n");
	err_occurred = true;
    }

	if (wxGetApp().frame->rbt_SomeStartNodes->GetValue() && wxGetApp().frame->lbx_StartNodes->IsEmpty()) {
	errors << _T("Start vertices are set to 'some', but list is empty.\n");
	err_occurred = true;
	}

	if (wxGetApp().frame->rbt_SomeEndNodes->GetValue() && wxGetApp().frame->lbx_EndNodes->IsEmpty()) {
	errors << _T("End vertices are set to 'some', but list is empty.\n");
	err_occurred = true;
	}

	if (err_occurred) {
		wxMessageDialog err_msg(wxGetApp().frame, errors, wxT("Some errors occured"), wxOK | wxICON_WARNING);
		err_msg.ShowModal();
		return false;
	}
	return true;
}
