#include <fstream>
#include <iostream>
#include <set>
#include <string>
#include <vector>

#include <math.h>
#include <stdio.h>
#include <time.h>

#ifdef __unix__
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif

#include "searchthread.h"
#include "resultlist.h"
#include "guiApp.h"
#include "guiFrm.h"
#include "../debug.h"
#include "../find_path.h"
#include "../graph.h"
#include "../problem.h"
#include "../util.h"
#include "../trial.h"

DEFINE_EVENT_TYPE(EVT_CREATE_LIST)
DEFINE_EVENT_TYPE(EVT_ADD_GRAPH_TAB)
DEFINE_EVENT_TYPE(EVT_ADD_RESLIST_TAB)

SearchThread::SearchThread(ParamSet *paramset) : param(paramset) {
}

std::set<vertex_t> read_vertex_list(const bool start_end, const Graph& g) {
    std::string line;
    std::size_t lineno = 0;
    std::set<vertex_t> vertices;
    
    std::size_t num_vert = start_end ? wxGetApp().frame->lbx_StartNodes->GetCount() : 
                wxGetApp().frame->lbx_EndNodes->GetCount();
    
    for( lineno = 0; lineno < num_vert; lineno++) {
    
    if (start_end) line = wxGetApp().frame->lbx_StartNodes->GetString(lineno).mb_str(wxConvUTF8);
    else line = wxGetApp().frame->lbx_EndNodes->GetString(lineno).mb_str(wxConvUTF8);
	const vertex_t* pv = g.lookup_vertex(line);
	if (!pv) {
	    std::cerr << lineno << ": warning: unknown vertex '" << line << "'\n"; 
	} else if (vertices.find(*pv) != vertices.end()) {
	    std::cerr << lineno << ": warning: ignoring duplicate vertex '"
		      << line << "'\n";
	} else {
	    vertices.insert(*pv);
	}
    }
    return vertices;
}


void *SearchThread::Entry()
{
    Problem problem;
	Graph g = *(wxGetApp().g);

    if (param->m_num_trials == 0)
	param->m_num_trials = trials_for_prob(param->m_path_length, param->m_num_colors, param->m_success_prob / 100);

    std::vector<bool> is_start_vertex(g.num_vertices());
    if (wxGetApp().frame->rbt_SomeStartNodes->GetValue() == true) {
	std::set<vertex_t> start_vertices = read_vertex_list(true, g);
	for (std::set<vertex_t>::const_iterator it = start_vertices.begin();
		it != start_vertices.end(); ++it)
		is_start_vertex[*it] = true;
    } else {
	for (std::size_t i = 0; i < g.num_vertices(); ++i)
	    is_start_vertex[i] = true;
    }

    std::vector<bool> is_end_vertex(g.num_vertices());
    if (wxGetApp().frame->rbt_SomeEndNodes->GetValue() == true) {
	std::set<vertex_t> end_vertices = read_vertex_list(false, g);
	for (std::set<vertex_t>::const_iterator it = end_vertices.begin();
		it != end_vertices.end(); ++it)
		is_end_vertex[*it] = true;
    } else {
	for (std::size_t i = 0; i < g.num_vertices(); ++i)
	    is_end_vertex[i] = true;
    }

    std::size_t max_common = int(param->m_path_length * (param->m_filter / 100));
       
    problem.g = g;
    problem.is_start_vertex = is_start_vertex;
    problem.is_end_vertex = is_end_vertex;
    problem.path_length = param->m_path_length;
    problem.num_preheat_trials = param->m_preheat_trials;
    problem.num_trials = param->m_num_trials;
    problem.success_prob = param->m_success_prob / 100;
    problem.num_colors = param->m_num_colors;

    PathSet *paths = new PathSet(lightest_path(problem, param->m_num_paths, max_common, param->m_mode, param->m_max_lb_edges, this));
 
	wxMutexGuiEnter();
	if (paths->size() > 0) {
        wxGetApp().frame->resListDone = false;
		wxCommandEvent evtAddGraphTab(EVT_ADD_GRAPH_TAB);
        wxPostEvent(wxGetApp().frame, evtAddGraphTab);
		wxCommandEvent evtAddResListTab(EVT_ADD_RESLIST_TAB);
        wxPostEvent(wxGetApp().frame, evtAddResListTab);
		wxMutexGuiLeave();

		Yield();
		while(wxGetApp().frame->resListDone == false) {
			wxThread::Sleep(200);
		}

		wxMutexGuiEnter();
		int sel = wxGetApp().frame->ntb_Results->GetPageCount() - 1;
		if (sel == -1) return 0;
		ResultList* res_list = wxGetApp().frame->ResPathList[sel];
		res_list->resModIsSPD = wxGetApp().modIsSPD;
		res_list->paths = paths;
		res_list->g = wxGetApp().g;
		res_list->graph_filename = wxGetApp().graph_filename;

		wxCommandEvent evtCreateList(EVT_CREATE_LIST);
		wxPostEvent(wxGetApp().frame->ResPathList[sel], evtCreateList);
		wxGetApp().frame->StatusBar->SetStatusText(wxT("Ready"), 1);
	} else {
		wxGetApp().frame->StatusBar->SetStatusText(wxT("No paths found!"), 1);
	}
	wxGetApp().frame->btn_SearchPaths->SetLabel(wxT("Search"));
	wxGetApp().frame->btn_StopSearch->Disable();
	wxGetApp().frame->search_status = 0;
	wxMutexGuiLeave();

	return NULL;
}
